import logging

from fastgames import Server, Dispatcher, Handler


def main():
    ip_address = ("192.168.31.25", 1235)

    server = Server(ip_address=ip_address)

    dispatcher = Dispatcher(
        handlers={
            "privet": Handler(function=lambda: print("privet"))
        }
    )

    server.get_clients(dispatcher)


if __name__ == "__main__":
    logging.basicConfig(level=logging.DEBUG)

    main()



from abc import ABC, abstractmethod
from typing import Any

from .handler import Handler


class BaseMiddleware(ABC):
    @abstractmethod
    def __call__(self, handler: Handler, data: dict[str, Any]):
        pass



from typing import Any, Optional

from pydantic import BaseModel

from .models import RequestModel, ResponseModel
from .handler import Handler
from .middleware import BaseMiddleware
from .router import Router


class Dispatcher:
    def __init__(
            self,
            handlers: Optional[dict[str, Handler]] = None,
            middlewares: Optional[list[BaseMiddleware]] = None
    ):
        self.__handlers: dict[str, Handler] = handlers or {}
        self.__middlewares: list[BaseMiddleware] = middlewares or []

    def include_routers(self, *args: Router) -> None:
        """Добавляет handlers из Routers в Dispatcher"""
        for router in args:
            self.__handlers.update(router.handlers)

    def register_middlewares(self, *args: BaseMiddleware) -> None:
        """Добавляет middlewares в Dispatcher"""
        for middleware in args:
            self.__middlewares.append(middleware)

    def handle_data(self, data: list[dict[str, Any]]) -> dict[str, Any]:
        """Обработка полученных данных от клиента"""
        response: dict[str, Any] = {}

        for request in [RequestModel(**item) for item in data]:
            if (handler := self.__handlers.get(request.name, None)) is None:
                continue

            data: dict[str, Any] = {}

            data.update(
                {
                    name: parameter.annotation(**request.data)
                    for name, parameter in handler.parameters.items()
                    if issubclass(parameter.annotation, BaseModel)
                }
            )

            for middleware in self.__middlewares:
                middleware(handler=handler, data=data)

            result = handler(data=data)

            isinstance(result, ResponseModel) and response.update({result.name: result.data})

        return response



from logging import getLogger
from typing import Any
from socket import socket, AF_INET, SOCK_STREAM
from json import loads, dumps
from threading import Thread
from time import sleep

from .dispatcher import Dispatcher


logger = getLogger(name=__name__)


class _Client:
    def __init__(self, client_data: tuple[socket, Any]):
        self.__socket, self.__address = client_data

        logger.info(f"New connect: {self.__address}")

    def __call__(self, dispatcher: Dispatcher) -> None:
        """Клиентский цикл, который открыт в отдельном потоке"""
        while True:
            try:
                data = loads(self.__socket.recv(1024).decode('UTF-8'))
            except BlockingIOError:
                continue
            except Exception:
                logger.warning("Error when receiving data from the client")
                break

            try:
                result = dispatcher.handle_data(data=data)
            except Exception:
                logger.warning("Client data processing error")
                break

            try:
                self.__socket.sendall(bytes(dumps(result), 'UTF-8'))
            except Exception:
                logger.warning("Error when sending data to the client")
                break

        self.__socket.close()
        logger.info(f'Disconnect: {self.__address}')


class Server:
    def __init__(self, ip_address: tuple[str, int]):
        self.__socket = socket(AF_INET, SOCK_STREAM)
        self.__socket.bind(ip_address)
        self.__socket.listen(10)
        self.__socket.setblocking(False)

        logger.info(f"Create server: {ip_address}")

    def get_clients(self, dispatcher: Dispatcher) -> None:
        """Ожидает подключения к серверу и запускает отдельный поток для клиента"""
        try:
            while True:
                try:
                    Thread(
                        target=_Client(client_data=self.__socket.accept()),
                        args=(dispatcher,)
                    ).start()
                except BlockingIOError:
                    sleep(0.1)
        except KeyboardInterrupt:
            self.__socket.close()
            logger.info(f"Close server...")



import socket
import json
from typing import Any


def main():
    ip_address = ("192.168.31.25", 1235)

    client = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
    client.connect(ip_address)

    requests = [dict(name="privet")]

    client.sendall(bytes(json.dumps(requests), 'UTF-8'))

    try:
        responses: dict[str, Any] = json.loads((client.recv(1024)).decode('utf-8'))
    except Exception:
        responses = {}

    print(responses.get("privet"))


if __name__ == "__main__":
    main()



from typing import Any, Optional

from pydantic import BaseModel


class RequestModel(BaseModel):
    name: str
    data: Optional[dict[str, Any]] = {}


class ResponseModel(BaseModel):
    name: str
    data: Any



from .dispatcher import (
    Router,
    Dispatcher,
    Handler,
    BaseMiddleware
)
from .base import Server


__all__ = [
    "Server",
    "Router",
    "Dispatcher",
    "Handler",
    "BaseMiddleware"
]



from typing import Callable, Any, Optional
from inspect import signature

from .models import ResponseModel


class Handler:
    def __init__(self, function: Callable):
        self.__function = function
        self.__signature = signature(self.__function)

        self.parameters = self.__signature.parameters

    def __call__(self, data: dict[str, Any]) -> Optional[ResponseModel]:
        return self.__function(
            **{
                name: parameter
                for name, parameter in data.items()
                if self.parameters.get(name, None) is not None
            }
        )



from setuptools import setup, find_packages

version = "0.0.4"


setup(
    name='fastgames',
    version=version,
    description='Fast creation of python games',
    author='Borov777py',
    author_email='kozhushko756@mail.ru',
    url='https://github.com/Borov777py/fastgame',
    packages=find_packages(),
    install_requires=[
        'pydantic',
    ]
)



from typing import Callable

from .handler import Handler


class Router:
    def __init__(self):
        self.handlers: dict[str, Handler] = {}

    def check_request(self, name: str):
        def decorator(function: Callable):
            self.handlers[name] = Handler(function=function)

        return decorator
